{-# LANGUAGE CPP #-}

module FrontendPaths (getFilesDir) where

#if defined(linux_HOST_OS)

import System.Environment.XDG.BaseDir (getUserDataDir)

-- | Get the path to a directory that is specific to the application where
-- internal application data may be written (and read back).
getFilesDir :: IO (Maybe FilePath)
getFilesDir =
    Just <$>
         -- It's not likely this is really the "mobile" case but we want to
         -- avoid colliding with PrivateStorageDesktop which uses
         -- "privatestorage".
         getUserDataDir "privatestoragemobile"

#else

import qualified Android.HaskellActivity as Android

-- | Get the path to a directory that is private to the application where
-- internal application data may be written (and read back).
--
-- The application requires no special permissions to use this directory.  If
-- the app is uninstalled the directory and its children will be deleted.
--
-- https://developer.android.com/reference/android/content/Context#getFilesDir()
getFilesDir :: IO (Maybe FilePath)
getFilesDir = Android.getHaskellActivity >>= Android.getFilesDir

#endif
